# Configuration file for the Sphinx documentation builder.
# pylint: disable=invalid-name,redefined-builtin,wrong-import-position,import-error,unused-import
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

import myst_parser

# from recommonmark.transform import AutoStructify

sys.path.insert(0, os.path.abspath(".."))


import strenum

# -- Project information -----------------------------------------------------

project = "StrEnum"
copyright = "2021, James Sinclair"
author = "James Sinclair"

# The full version, including alpha/beta/rc tags
release = strenum.__version__
github_doc_root = "https://github.com/irgeek/strenum/tree/master/docs/"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    # "sphinx.ext.autosectionlabel",
    "sphinx.ext.autodoc",
    # "sphinx.ext.viewcode",
    # "sphinx.ext.autosummary",
    "myst_parser",
]

source_suffix = [".rst", ".md"]
master_doc = "index"
templates_path = []

html_theme = "sphinx_rtd_theme"
html_static_path = []

exclude_patterns = [".venv"]

pygments_style = "sphinx"

autodoc_member_order = "bysource"
autodoc_class_signature = "separated"
autosummary_generate = True

myst_enable_extensions = [
    "colon_fence",
    "linkify",
]



# pylint:disable=missing-docstring,invalid-name
from enum import auto
import pytest
from strenum import StrEnum


class HttpMethod(StrEnum):
    GET = auto()
    HEAD = auto()
    POST = auto()
    PUT = auto()
    DELETE = auto()
    CONNECT = auto()
    OPTIONS = auto()
    TRACE = auto()
    PATCH = auto()


def test_isinstance_str():
    assert isinstance(HttpMethod.GET, str)


def test_value_isinstance_str():
    assert isinstance(HttpMethod.GET.value, str)


def test_str_builtin():
    assert str(HttpMethod.GET) == "GET"


def test_str_equals():
    assert HttpMethod.GET == "GET"


def test_str_hash():
    assert hash(HttpMethod.GET) == hash("GET")


def test_str_cmp():
    assert HttpMethod.GET > "FET"
    assert HttpMethod.GET < "GETA"


def test_nonstring_fails():
    # pylint:disable=unused-variable
    with pytest.raises(TypeError):

        class BadEnum(StrEnum):
            ONE = 1
            TWO = 2



# pylint:disable=missing-docstring,invalid-name
from enum import auto
from strenum import LowercaseStrEnum, UppercaseStrEnum


def test_lowercase_auto():
    class BeamType(LowercaseStrEnum):
        START = auto()
        STOP = auto()
        PARTIAL = auto()

    assert BeamType.START == "start"
    assert BeamType.STOP == "stop"
    assert BeamType.PARTIAL == "partial"


def test_uppercase_auto():
    class HttpMethod(UppercaseStrEnum):
        Get = auto()
        Head = auto()
        Post = auto()
        Put = auto()
        Delete = auto()
        Connect = auto()
        Options = auto()
        Trace = auto()
        Patch = auto()

    assert HttpMethod.Get == "GET"
    assert HttpMethod.Head == "HEAD"
    assert HttpMethod.Post == "POST"
    assert HttpMethod.Put == "PUT"
    assert HttpMethod.Delete == "DELETE"
    assert HttpMethod.Connect == "CONNECT"
    assert HttpMethod.Options == "OPTIONS"
    assert HttpMethod.Trace == "TRACE"
    assert HttpMethod.Patch == "PATCH"



# pylint: disable=missing-docstring,too-many-arguments
import pytest
from strenum import _NameMangler


word_split_test_data = (
    ("one", ["one"]),
    ("one two", ["one", "two"]),
    ("one two three", ["one", "two", "three"]),
    ("ONETwoThree", ["ONE", "Two", "Three"]),
    ("OneTWOThree", ["One", "TWO", "Three"]),
    ("OneTwoTHREE", ["One", "Two", "THREE"]),
    ("fromCamelCase", ["from", "Camel", "Case"]),
    ("FromPascalCase", ["From", "Pascal", "Case"]),
    ("from-kebab-case", ["from", "kebab", "case"]),
    ("from_snake_case", ["from", "snake", "case"]),
    ("FROM_MACRO_CASE", ["FROM", "MACRO", "CASE"]),
    ("from_Camel_Snake_Case", ["from", "Camel", "Snake", "Case"]),
    ("From_Pascal_Snake_Case", ["From", "Pascal", "Snake", "Case"]),
    ("FROM-COBOL-CASE", ["FROM", "COBOL", "CASE"]),
    ("From-Http-Header-Case", ["From", "Http", "Header", "Case"]),
)


@pytest.mark.parametrize("word,expected", word_split_test_data)
def test_word_splitting(word, expected):
    name_mangler = _NameMangler()
    assert list(name_mangler.words(word)) == expected


test_data = [
    ("camel", "oneTwoThree"),
    ("pascal", "OneTwoThree"),
    ("kebab", "one-two-three"),
    ("snake", "one_two_three"),
    ("macro", "ONE_TWO_THREE"),
    ("camel_snake", "one_Two_Three"),
    ("pascal_snake", "One_Two_Three"),
    ("spongebob", "ONE_twO_ThREe"),
    ("cobol", "ONE-TWO-THREE"),
    ("http_header", "One-Two-Three"),
]


@pytest.mark.parametrize("func_name,expected", test_data)
def test_name_mangler(func_name, expected):
    name_mangler = _NameMangler()

    func = getattr(name_mangler, func_name)
    assert func("one two three") == expected



# pylint:disable=missing-docstring,invalid-name,abstract-method
import sys
from enum import auto
import pytest

from strenum import KebabCaseStrEnum
from strenum.mixins import Comparable


@pytest.mark.skipif(
    sys.version_info < (3, 7, 2), reason="incompatible with Python before 3.7.2"
)
def test_comparable():
    class HttpHeader(Comparable, KebabCaseStrEnum):
        ContentType = auto()
        Host = auto()
        Accept = auto()
        XForwardedFor = auto()

        def _cmp_values(self, other):
            return self.value.lower(), str(other).lower()

    assert HttpHeader.ContentType == "Content-Type"
    assert HttpHeader.ContentType == "content-type"
    assert HttpHeader.ContentType == "coNtEnt-tyPe"

    assert HttpHeader.ContentType != "content-types"

    assert HttpHeader.ContentType < "d"
    assert HttpHeader.ContentType > "b"

    assert HttpHeader.ContentType <= "d"
    assert HttpHeader.ContentType <= "Content-Type"
    assert HttpHeader.ContentType >= "b"
    assert HttpHeader.ContentType >= "Content-Type"


@pytest.mark.skipif(
    sys.version_info < (3, 7, 2), reason="incompatible with Python before 3.7.2"
)
def test_comparable_not_implemented():
    class HttpHeader(Comparable, KebabCaseStrEnum):
        ContentType = auto()
        Host = auto()
        Accept = auto()
        XForwardedFor = auto()

    with pytest.raises(NotImplementedError):
        assert HttpHeader.ContentType == "content-type"



# pylint:disable=missing-docstring,invalid-name
from enum import auto
from strenum import (
    CamelCaseStrEnum,
    PascalCaseStrEnum,
    KebabCaseStrEnum,
    SnakeCaseStrEnum,
    MacroCaseStrEnum,
    CamelSnakeCaseStrEnum,
    PascalSnakeCaseStrEnum,
    SpongebobCaseStrEnum,
    CobolCaseStrEnum,
    HttpHeaderCaseStrEnum,
)


def test_camel_case_auto():
    class TestEnum(CamelCaseStrEnum):
        One = auto()
        OneTwo = auto()
        OneTwoThree = auto()

    assert TestEnum.One == "one"
    assert TestEnum.OneTwo == "oneTwo"
    assert TestEnum.OneTwoThree == "oneTwoThree"


def test_pascal_case_auto():
    class TestEnum(PascalCaseStrEnum):
        One = auto()
        OneTwo = auto()
        OneTwoThree = auto()

    assert TestEnum.One == "One"
    assert TestEnum.OneTwo == "OneTwo"
    assert TestEnum.OneTwoThree == "OneTwoThree"


def test_kebab_case_auto():
    class TestEnum(KebabCaseStrEnum):
        One = auto()
        OneTwo = auto()
        OneTwoThree = auto()

    assert TestEnum.One == "one"
    assert TestEnum.OneTwo == "one-two"
    assert TestEnum.OneTwoThree == "one-two-three"


def test_snake_case_auto():
    class TestEnum(SnakeCaseStrEnum):
        One = auto()
        OneTwo = auto()
        OneTwoThree = auto()

    assert TestEnum.One == "one"
    assert TestEnum.OneTwo == "one_two"
    assert TestEnum.OneTwoThree == "one_two_three"


def test_macro_case_auto():
    class TestEnum(MacroCaseStrEnum):
        One = auto()
        OneTwo = auto()
        OneTwoThree = auto()

    assert TestEnum.One == "ONE"
    assert TestEnum.OneTwo == "ONE_TWO"
    assert TestEnum.OneTwoThree == "ONE_TWO_THREE"


def test_camel_snake_case_auto():
    class TestEnum(CamelSnakeCaseStrEnum):
        One = auto()
        OneTwo = auto()
        OneTwoThree = auto()

    assert TestEnum.One == "one"
    assert TestEnum.OneTwo == "one_Two"
    assert TestEnum.OneTwoThree == "one_Two_Three"


def test_pascal_snake_case_auto():
    class TestEnum(PascalSnakeCaseStrEnum):
        One = auto()
        OneTwo = auto()
        OneTwoThree = auto()

    assert TestEnum.One == "One"
    assert TestEnum.OneTwo == "One_Two"
    assert TestEnum.OneTwoThree == "One_Two_Three"


def test_spongebob_case_auto():
    class TestEnum(SpongebobCaseStrEnum):
        One = auto()
        OneTwo = auto()
        OneTwoThree = auto()

    assert TestEnum.One == "one"
    assert TestEnum.OneTwo == "one_TWo"
    assert TestEnum.OneTwoThree == "one_TWo_thrEE"


def test_cobol_case_auto():
    class TestEnum(CobolCaseStrEnum):
        One = auto()
        OneTwo = auto()
        OneTwoThree = auto()

    assert TestEnum.One == "ONE"
    assert TestEnum.OneTwo == "ONE-TWO"
    assert TestEnum.OneTwoThree == "ONE-TWO-THREE"


def test_http_header_case_auto():
    class TestEnum(HttpHeaderCaseStrEnum):
        One = auto()
        OneTwo = auto()
        OneTwoThree = auto()

    assert TestEnum.One == "One"
    assert TestEnum.OneTwo == "One-Two"
    assert TestEnum.OneTwoThree == "One-Two-Three"
